/*
Copyright (C) 1997-2001 Id Software, Inc.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  

See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/

#include "extdll.h"
#include "basemenu.h"
#include "utils.h"
#include "../cl_dll/kbutton.h"
#include "menu_btnsbmp_table.h"

#define ART_BANNER			"gfx/shell/head_advanced"

#define ID_BACKGROUND		0
#define ID_BANNER			1

#define ID_DONE				2
#define ID_PAGES			128

#define MAX_SPINBOXES 16
#define MAX_FIELDS 32 // STRING and NUMBER tags
#define MAX_CHECKBOXES 16

#define MAX_SPINBOX_ELEMENTS 64

typedef struct
{
	menuFramework_s	menu;
	int				checkbox;
	int				field;
	int				spinControl;
	int				firstLastY;
	int				secondLastY;
	int				pagesNum;
	int				page[16+32+16];

	menuBitmap_s	background;
	menuBitmap_s	banner;

	menuPicButton_s	done;

	menuSpinControl_s pages;

	menuCheckBox_s	checkboxes[MAX_CHECKBOXES];
	menuField_s		fields[MAX_FIELDS];
	menuSpinControl_s spinControls[MAX_SPINBOXES];
} uiAdvOptions_t;
/* // adamix, will keep it for testing
void logprintf(char * szFormat, ...)
{
		// Collect the function arguments
		va_list args;
		char szLogString[512];
		va_start(args, szFormat);
		vsprintf(szLogString, szFormat, args);
		va_end(args);

		// Open the log file
		FILE * fLog = fopen("test.log", "a");

		if(fLog)
		{
			// Print the log string to the log file
			fprintf(fLog, "%s\n", szLogString);

			// Close the log file
			fclose(fLog);
		}
}
*/

typedef struct
{
	char * name;
	char * value;
} listElem_t;

typedef struct
{
	listElem_t elements[MAX_SPINBOX_ELEMENTS];
	int		   elemNum;
} listElems_t;

static uiAdvOptions_t	uiAdvOptions;
static listElems_t  listfields[MAX_SPINBOXES];

static int GetPageForElement( void )
{

}

static menuCommon_s * GetElementByElementID( int id )
{
	int i = 0;
	for(i = 0; i < uiAdvOptions.checkbox; i++)
	{
		if(uiAdvOptions.checkboxes[i].generic.id2 == id)
			return &uiAdvOptions.checkboxes[i].generic;
	}
	for(i = 0; i < uiAdvOptions.field; i++)
	{
		if(uiAdvOptions.fields[i].generic.id2 == id)
			return &uiAdvOptions.fields[i].generic;
	}
	for(i = 0; i < uiAdvOptions.spinControl; i++)
	{
		if(uiAdvOptions.spinControls[i].generic.id2 == id)
			return &uiAdvOptions.spinControls[i].generic;
	}
	return NULL;
}

/*
=================
UI_AdvControls_UpdateConfig
=================
*/
static void UI_AdvOptions_UpdateConfig( void )
{

}

/*
=================
UI_AdvControls_GetConfig
=================
*/
static void UI_AdvOptions_GetConfig( void )
{

}

/*
=================
UI_AdvControls_Callback
=================
*/
static void UI_AdvOptions_Callback( void *self, int event )
{
	menuCommon_s	*item = (menuCommon_s *)self;

	if(item->type == QMTYPE_CHECKBOX)
	{
		if( event == QM_PRESSED )
			((menuCheckBox_s *)self)->focusPic = UI_CHECKBOX_PRESSED;
		else ((menuCheckBox_s *)self)->focusPic = UI_CHECKBOX_FOCUS;
	}

	if( event == QM_CHANGED )
	{
		if(item->type == QMTYPE_CHECKBOX)
		{
			CVAR_SET_FLOAT(item->cvar, ((menuCheckBox_s *)self)->enabled);
		}
		if(item->type == QMTYPE_FIELD)
		{
			if(item->flags & QMF_NUMBERSONLY)
			{
				CVAR_SET_FLOAT(item->cvar, atoi(((menuField_s *)self)->buffer));
			}
			else
			{
				CVAR_SET_STRING(item->cvar, ((menuField_s *)self)->buffer);
			}
		}
		if(item->type == QMTYPE_SPINCONTROL && item->id != ID_PAGES)
		{
			menuSpinControl_s* control = (menuSpinControl_s *)self;
			
			int value = control->curValue;
			control->generic.name = listfields[control->generic.id].elements[value].name;

			CVAR_SET_FLOAT(item->cvar, atoi(listfields[control->generic.id].elements[value].value));
		}
		else if(item->type == QMTYPE_SPINCONTROL && item->id == ID_PAGES)
		{
			menuSpinControl_s* pageControl = (menuSpinControl_s *)self;
			static char page[8];
			sprintf(page, "%.f", pageControl->curValue);
			pageControl->generic.name = page;

			uiAdvOptions.firstLastY = 270;
			uiAdvOptions.secondLastY = 270;
			for(int i = 0; i <= (uiAdvOptions.checkbox + uiAdvOptions.field + uiAdvOptions.spinControl); i++)
			{
				menuCommon_s * control = GetElementByElementID(i);
				if(control == NULL) continue;
				if(uiAdvOptions.page[i] != (int)pageControl->curValue)
				{
					control->flags |= QMF_HIDDEN;
				}
				else if(uiAdvOptions.page[i] == (int)pageControl->curValue)
				{

					control->x = 335;
					control->y = uiAdvOptions.firstLastY;
					uiAdvOptions.firstLastY = uiAdvOptions.firstLastY + 80;

					if(control->type == QMTYPE_SPINCONTROL)
					{
						control->width = 168;
						control->height = 26;
						UI_SpinControl_Init((menuSpinControl_s*)control);
					}
					control->flags &= ~QMF_HIDDEN;
				}
			}
		}
		return;
	}

	if( event != QM_ACTIVATED )
		return;

	switch( item->id )
	{
	case ID_DONE:
		UI_PopMenu();
		break;
	}
}

/*
=================
UI_AdvControls_Init
=================
*/
static void UI_AdvOptions_Init( void )
{
	memset( &uiAdvOptions, 0, sizeof( uiAdvOptions_t ));
	memset( &listfields, 0, sizeof(listfields));
	uiAdvOptions.firstLastY = 230;
	uiAdvOptions.secondLastY = 230;
	int pages = 1;

	uiAdvOptions.menu.vidInitFunc = UI_AdvOptions_Init;

	uiAdvOptions.background.generic.id = ID_BACKGROUND;
	uiAdvOptions.background.generic.type = QMTYPE_BITMAP;
	uiAdvOptions.background.generic.flags = QMF_INACTIVE;
	uiAdvOptions.background.generic.x = 0;
	uiAdvOptions.background.generic.y = 0;
	uiAdvOptions.background.generic.width = 1024;
	uiAdvOptions.background.generic.height = 768;
	uiAdvOptions.background.pic = ART_BACKGROUND;

	uiAdvOptions.banner.generic.id = ID_BANNER;
	uiAdvOptions.banner.generic.type = QMTYPE_BITMAP;
	uiAdvOptions.banner.generic.flags = QMF_INACTIVE|QMF_DRAW_ADDITIVE;
	uiAdvOptions.banner.generic.x = UI_BANNER_POSX;
	uiAdvOptions.banner.generic.y = UI_BANNER_POSY;
	uiAdvOptions.banner.generic.width = UI_BANNER_WIDTH;
	uiAdvOptions.banner.generic.height = UI_BANNER_HEIGHT;
	uiAdvOptions.banner.pic = ART_BANNER;

	uiAdvOptions.done.generic.id = ID_DONE;
	uiAdvOptions.done.generic.type = QMTYPE_BM_BUTTON;
	uiAdvOptions.done.generic.flags = QMF_HIGHLIGHTIFFOCUS|QMF_DROPSHADOW; 
	uiAdvOptions.done.generic.x = 72;
	uiAdvOptions.done.generic.y = 230;
	uiAdvOptions.done.generic.name = "Done";
	uiAdvOptions.done.generic.statusText = "Go back to the Multiplayer Menu";
	uiAdvOptions.done.generic.callback = UI_AdvOptions_Callback;

	UI_UtilSetupPicButton( &uiAdvOptions.done, PC_DONE );

	uiAdvOptions.pages.generic.id = ID_PAGES;
	uiAdvOptions.pages.generic.type = QMTYPE_SPINCONTROL;
	uiAdvOptions.pages.generic.flags = QMF_CENTER_JUSTIFY|QMF_HIGHLIGHTIFFOCUS|QMF_DROPSHADOW;
	uiAdvOptions.pages.generic.x = 800;
	uiAdvOptions.pages.generic.y = 200;
	uiAdvOptions.pages.generic.width = 168;
	uiAdvOptions.pages.generic.height = 26;
	uiAdvOptions.pages.generic.callback = UI_AdvOptions_Callback;
	uiAdvOptions.pages.generic.statusText = "Select page";
	uiAdvOptions.pages.minValue = 1;
	uiAdvOptions.pages.maxValue = 5;
	uiAdvOptions.pages.range  = 1;
	uiAdvOptions.pages.curValue = 1;
	uiAdvOptions.pages.generic.name = "1";


	UI_AddItem( &uiAdvOptions.menu, (void *)&uiAdvOptions.background );
	UI_AddItem( &uiAdvOptions.menu, (void *)&uiAdvOptions.banner );
	UI_AddItem( &uiAdvOptions.menu, (void *)&uiAdvOptions.done );
	UI_AddItem( &uiAdvOptions.menu, (void *)&uiAdvOptions.pages );

	char *afile = (char *)LOAD_FILE( "settings.scr", NULL );
	char *pfile = afile;
	char token[1024];

	if( !afile )
	{
		// not error, not warning, just notify
		Con_Printf( "UI_AdvOptions_Init: settings.scr not found\n" );
		return;
	}

	bool bRun = true;
	int elem = 0;
	while(bRun)
	{
		static char setting[64][512];
		static char text[64][512];
		static char type[64][512];
		static char defvalue[64][512];
		static char values[64][512];

		strcpy((char*)&values[elem], "");

		for(int i = 0; i < 4; i++)
		{
			pfile = COM_ParseFile( pfile, token );
			if( !pfile ) 
			{
				bRun = false;
				break;
			}
			bool bCheck = true;
			while(bCheck)
			{
				if( token[0] == '{' || token[0] == '}' || token[0] == ')' || token[0] == '(' || token[0] == '\'' || token[0] == ',' ) 
				{
					pfile = COM_ParseFile( pfile, token );
					if( !pfile ) 
					{
						bRun = false;
						break;
					}
				}
				else
				{
					break;
				}
			}
			if(i == 0)
			{
				strcpy(setting[elem], token);
			}
			else if(i == 1)
			{
				strcpy(text[elem], token);
			}
			else if(i == 2)
			{
				strcpy((char*)&type[elem], token);
				if( !stricmp( token, "NUMBER" ))
				{
					for(int i = 0; i < 2; i++)
					{
						pfile = COM_ParseFile( pfile, token );
						if( !pfile ) 
						{
							bRun = false;
							break;
						}
						sprintf((char*)&values[elem], "%s %s", (char*)&values[elem], token);
					}
				}
				if( !stricmp( token, "LIST" ))
				{
					static int pos = 0;
					while(token[0] != '}')
					{
						pfile = COM_ParseFile( pfile, token );
						if( !pfile ) 
						{
							bRun = false;
							break;
						}

						if(token[0] != '}')
						{
							sprintf((char*)&values[elem], "%s\n%s", (char*)&values[elem], token);
						}
					}
					char * pch = strtok(values[elem], "\n");
					while(pch != NULL)
					{
						listfields[uiAdvOptions.spinControl].elements[listfields[uiAdvOptions.spinControl].elemNum].name = pch;
						pch = strtok(NULL, "\n");
						if(!pch) break;
						listfields[uiAdvOptions.spinControl].elements[listfields[uiAdvOptions.spinControl].elemNum].value = pch;
						listfields[uiAdvOptions.spinControl].elemNum++;
						pch = strtok(NULL, "\n");
						if(listfields[uiAdvOptions.spinControl].elemNum == MAX_SPINBOX_ELEMENTS) break;
					}
					listfields[uiAdvOptions.spinControl].elemNum--;
				}
			}
			else if(i == 3)
			{
				strcpy(defvalue[elem], token);
			}

		}
		if(bRun != false)
		{
			if(elem <= 5)
				uiAdvOptions.page[elem] = 1;
			else if(elem <= 10)
				uiAdvOptions.page[elem] = 2;
			else if(elem <= 15)
				uiAdvOptions.page[elem] = 3;
			else if(elem <= 20)
				uiAdvOptions.page[elem] = 4;
			if( !stricmp( (char*)&type[elem], "BOOL" ))
			{
				if(uiAdvOptions.checkbox >= MAX_CHECKBOXES) continue;
				uiAdvOptions.firstLastY = uiAdvOptions.firstLastY + 40;
				uiAdvOptions.checkboxes[uiAdvOptions.checkbox].generic.id = uiAdvOptions.checkbox;
				uiAdvOptions.checkboxes[uiAdvOptions.checkbox].generic.id2 = elem;
				uiAdvOptions.checkboxes[uiAdvOptions.checkbox].generic.type = QMTYPE_CHECKBOX;
				uiAdvOptions.checkboxes[uiAdvOptions.checkbox].generic.flags = QMF_HIGHLIGHTIFFOCUS|QMF_ACT_ONRELEASE|QMF_MOUSEONLY|QMF_DROPSHADOW;
				uiAdvOptions.checkboxes[uiAdvOptions.checkbox].generic.x = 300;
				uiAdvOptions.checkboxes[uiAdvOptions.checkbox].generic.y = uiAdvOptions.firstLastY;
				uiAdvOptions.checkboxes[uiAdvOptions.checkbox].generic.name = (const char*)text[elem];
				uiAdvOptions.checkboxes[uiAdvOptions.checkbox].generic.callback = UI_AdvOptions_Callback;
				uiAdvOptions.checkboxes[uiAdvOptions.checkbox].generic.statusText = (const char*)text[elem];
				uiAdvOptions.checkboxes[uiAdvOptions.checkbox].generic.cvar = (const char*)setting[elem];

				if(	CVAR_GET_FLOAT( setting[elem] ) )
					uiAdvOptions.checkboxes[uiAdvOptions.checkbox].enabled = 1;

				UI_AddItem( &uiAdvOptions.menu, (void *)&uiAdvOptions.checkboxes[uiAdvOptions.checkbox] );

				uiAdvOptions.checkbox++;
			}

			if( !stricmp( (char*)&type[elem], "NUMBER" ))
			{
				if(uiAdvOptions.field >= MAX_FIELDS) continue;
				uiAdvOptions.secondLastY = uiAdvOptions.secondLastY + 80;
				uiAdvOptions.fields[uiAdvOptions.field].generic.id = uiAdvOptions.field;
				uiAdvOptions.fields[uiAdvOptions.field].generic.id2 = elem;
				uiAdvOptions.fields[uiAdvOptions.field].generic.type = QMTYPE_FIELD;
				uiAdvOptions.fields[uiAdvOptions.field].generic.flags = QMF_HIGHLIGHTIFFOCUS|QMF_DROPSHADOW|QMF_NUMBERSONLY;
				uiAdvOptions.fields[uiAdvOptions.field].generic.x = 650;
				uiAdvOptions.fields[uiAdvOptions.field].generic.y = uiAdvOptions.secondLastY;
				uiAdvOptions.fields[uiAdvOptions.field].generic.width = 205;
				uiAdvOptions.fields[uiAdvOptions.field].generic.height = 32;
				uiAdvOptions.fields[uiAdvOptions.field].generic.name = (const char*)text[elem];
				uiAdvOptions.fields[uiAdvOptions.field].generic.callback = UI_AdvOptions_Callback;
				uiAdvOptions.fields[uiAdvOptions.field].generic.statusText = (const char*)text[elem];
				uiAdvOptions.fields[uiAdvOptions.field].generic.cvar = (const char*)setting[elem];

				uiAdvOptions.fields[uiAdvOptions.field].maxLength = 10;
				sprintf(uiAdvOptions.fields[uiAdvOptions.field].buffer, "%d", (int)CVAR_GET_FLOAT(setting[elem]));

				UI_AddItem( &uiAdvOptions.menu, (void *)&uiAdvOptions.fields[uiAdvOptions.field] );

				uiAdvOptions.field++;
			}

			if( !stricmp( (char*)&type[elem], "STRING" ))
			{
				if(uiAdvOptions.field >= MAX_FIELDS) continue;
				uiAdvOptions.secondLastY = uiAdvOptions.secondLastY + 80;
				uiAdvOptions.fields[uiAdvOptions.field].generic.id = uiAdvOptions.field;
				uiAdvOptions.fields[uiAdvOptions.field].generic.id2 = elem;
				uiAdvOptions.fields[uiAdvOptions.field].generic.type = QMTYPE_FIELD;
				uiAdvOptions.fields[uiAdvOptions.field].generic.flags = QMF_HIGHLIGHTIFFOCUS|QMF_DROPSHADOW;
				uiAdvOptions.fields[uiAdvOptions.field].generic.x = 650;
				uiAdvOptions.fields[uiAdvOptions.field].generic.y = uiAdvOptions.secondLastY;
				uiAdvOptions.fields[uiAdvOptions.field].generic.width = 205;
				uiAdvOptions.fields[uiAdvOptions.field].generic.height = 32;
				uiAdvOptions.fields[uiAdvOptions.field].generic.name = (const char*)text[elem];
				uiAdvOptions.fields[uiAdvOptions.field].generic.callback = UI_AdvOptions_Callback;
				uiAdvOptions.fields[uiAdvOptions.field].generic.statusText = (const char*)text[elem];
				uiAdvOptions.fields[uiAdvOptions.field].generic.cvar = (const char*)setting[elem];

				uiAdvOptions.fields[uiAdvOptions.field].maxLength = 128;
				strcpy(uiAdvOptions.fields[uiAdvOptions.field].buffer, CVAR_GET_STRING(setting[elem]));

				UI_AddItem( &uiAdvOptions.menu, (void *)&uiAdvOptions.fields[uiAdvOptions.field] );

				uiAdvOptions.field++;
			}

			if( !stricmp( (char*)&type[elem], "LIST" ))
			{
				if(uiAdvOptions.spinControl >= MAX_SPINBOXES) continue;
				uiAdvOptions.firstLastY = uiAdvOptions.firstLastY + 50;
				uiAdvOptions.spinControls[uiAdvOptions.spinControl].generic.id = uiAdvOptions.spinControl;
				uiAdvOptions.spinControls[uiAdvOptions.spinControl].generic.id2 = elem;
				uiAdvOptions.spinControls[uiAdvOptions.spinControl].generic.type = QMTYPE_SPINCONTROL;
				uiAdvOptions.spinControls[uiAdvOptions.spinControl].generic.flags = QMF_CENTER_JUSTIFY|QMF_HIGHLIGHTIFFOCUS|QMF_DROPSHADOW;
				uiAdvOptions.spinControls[uiAdvOptions.spinControl].generic.x = 335;
				uiAdvOptions.spinControls[uiAdvOptions.spinControl].generic.y = uiAdvOptions.firstLastY;
				uiAdvOptions.spinControls[uiAdvOptions.spinControl].generic.width = 168;
				uiAdvOptions.spinControls[uiAdvOptions.spinControl].generic.height = 26;
				uiAdvOptions.spinControls[uiAdvOptions.spinControl].generic.name = listfields[uiAdvOptions.spinControl].elements[0].name;
				uiAdvOptions.spinControls[uiAdvOptions.spinControl].generic.callback = UI_AdvOptions_Callback;
				uiAdvOptions.spinControls[uiAdvOptions.spinControl].generic.statusText = "";
				uiAdvOptions.spinControls[uiAdvOptions.spinControl].generic.cvar = (const char*)setting[elem];
				uiAdvOptions.spinControls[uiAdvOptions.spinControl].generic.statusText = (const char*)text[elem];
				uiAdvOptions.spinControls[uiAdvOptions.spinControl].minValue = 0;
				uiAdvOptions.spinControls[uiAdvOptions.spinControl].maxValue = listfields[uiAdvOptions.spinControl].elemNum;
				uiAdvOptions.spinControls[uiAdvOptions.spinControl].range = 1;
				
				for(int i = 0; i <= listfields[uiAdvOptions.spinControl].elemNum; i++)
				{
					if((int)CVAR_GET_FLOAT(setting[elem]) == atoi(listfields[uiAdvOptions.spinControl].elements[i].value))
					{
						uiAdvOptions.spinControls[uiAdvOptions.spinControl].generic.name = listfields[uiAdvOptions.spinControl].elements[i].name;
						break;
					}
				}

				UI_AddItem( &uiAdvOptions.menu, (void *)&uiAdvOptions.spinControls[uiAdvOptions.spinControl] );

				uiAdvOptions.spinControl++;
			}
			elem++;
		}
	}
	uiAdvOptions.firstLastY = 270;
	uiAdvOptions.secondLastY = 270;
	for(int i = 0; i <= (uiAdvOptions.checkbox + uiAdvOptions.field + uiAdvOptions.spinControl); i++)
	{
		menuCommon_s * control = GetElementByElementID(i);
		if(control == NULL) continue;
		if(uiAdvOptions.page[i] != 1)
		{
			control->flags |= QMF_HIDDEN;
		}
		else if(uiAdvOptions.page[i] == 1)
		{
			control->x = 335;
			control->y = uiAdvOptions.firstLastY;
			uiAdvOptions.firstLastY = uiAdvOptions.firstLastY + 80;

			if(control->type == QMTYPE_SPINCONTROL)
			{
				control->width = 168;
				control->height = 26;
				UI_SpinControl_Init((menuSpinControl_s*)control);
			}
		}
	}
	pages = ((uiAdvOptions.checkbox + uiAdvOptions.field + uiAdvOptions.spinControl)) / 5;
	uiAdvOptions.pages.maxValue = pages;
}

/*
=================
UI_AdvControls_Precache
=================
*/
void UI_AdvOptions_Precache( void )
{
	PIC_Load( ART_BACKGROUND );
	PIC_Load( ART_BANNER );
}

/*
=================
UI_AdvControls_Menu
=================
*/
void UI_AdvOptions_Menu( void )
{
	UI_AdvOptions_Precache();
	UI_AdvOptions_Init();

	UI_PushMenu( &uiAdvOptions.menu );
}
